
include std/iup.e
include std/iup_scintilla.e
include events.e

--**
-- Create a menu item.
--
public function menu_item( sequence handle, sequence text, atom action_cb, atom active = 1 )

	atom item = IupItem( text, NULL )
	IupSetCallback( item, "ACTION", action_cb )
	IupSetInt( item, "ACTIVE", active )
	IupSetHandle( handle, item )

	return item
end function

--**
-- Create a menu separator
--
public function menu_separator()
	return IupSeparator()
end function

--**
-- Create a toolbar item.
--
public function toolbar_item( sequence handle, sequence tip, sequence image, atom action_cb, atom active = 1 )

	atom item = IupButton()
	IupSetAttributes( item, "CANFOCUS=NO, FLAT=YES" )
	IupSetAttribute( item, "IMAGE", image )
	IupSetAttribute( item, "TIP", tip )
	IupSetInt( item, "ACTIVE", active )
	IupSetCallback( item, "ACTION", action_cb )
	IupSetHandle( handle, item )

	return item
end function

--**
-- Create a toolbar separator.
--
public function toolbar_separator()
	return IupSetAttributes( IupLabel(NULL), "SEPARATOR=VERTICAL" )
end function

--**
-- Create the File menu.
--
public function file_menu()

	atom menu = IupMenu({
		menu_item( "file_new",    "&New\t\tCtrl+N",   Icallback("new_cb"),    1 ),
		menu_item( "file_open",   "&Open...\tCtrl+O", Icallback("open_cb"),   1 ),
		menu_item( "file_save",   "&Save\t\tCtrl+S",  Icallback("save_cb"),   0 ),
		menu_item( "file_saveas", "Save &As..",       Icallback("saveas_cb"), 0 ),
		menu_item( "file_close",  "C&lose\t\tCtrl+W", Icallback("close_cb"),  0 ),
		menu_separator(),
		menu_item( "file_exit",   "E&xit\t\tAlt+F4",  Icallback("exit_cb"), 1 )
	})
	IupSetHandle( "file_menu", menu )

	return menu
end function

--**
-- Create the Edit menu.
--
public function edit_menu()

	atom menu = IupMenu({
		menu_item( "edit_undo", "&Undo\t\tCtrl+Z", Icallback("undo_cb"), 0 ),
		menu_item( "edit_redo", "&Redo\t\tCtrl+Y", Icallback("redo_cb"), 0 ),
		menu_separator(),
		menu_item( "edit_cut",    "Cu&t\t\tCtrl+X",   Icallback("cut_cb"),    0 ),
		menu_item( "edit_copy",   "&Copy\t\tCtrl+C",  Icallback("copy_cb"),   0 ),
		menu_item( "edit_paste",  "&Paste\t\tCtrl+V", Icallback("paste_cb"),  0 ),
		menu_item( "edit_delete", "&Delete",          Icallback("delete_cb"), 0 ),
		menu_separator(),
		menu_item( "edit_selectall", "Select &All\tCtrl+A", Icallback("selectall_cb"), 0 )
	})
	IupSetHandle( "edit_menu", menu )

	return menu
end function

--**
-- Create the Run menu.
--
public function run_menu()

	atom menu = IupMenu({
		menu_item( "run_run", "&Run\tF5", Icallback("run_cb") )
	})

	return menu
end function

--**
-- Create the Help menu.
--
public function help_menu()

	atom menu = IupMenu({
		menu_item( "help_layout", "Dialog Layout",  Icallback("layout_cb") ),
		menu_item( "help_about",  "&About...\tF12", Icallback("about_cb")  )
	})
	IupSetHandle( "help_menu", menu )

	return menu
end function

--**
-- Create the main menu.
--
public function main_menu()

	atom menu = IupMenu({
		IupSubmenu( "&File", file_menu() ),
		IupSubmenu( "&Edit", edit_menu() ),
		IupSubmenu( "&Run",  run_menu()  ),
		IupSubmenu( "&Help", help_menu() )
	})
	IupSetHandle( "menu", menu )

	return menu
end function

--**
-- Create the main toolbar.
--
public function main_toolbar()

	atom toolbar = IupHbox({
		toolbar_item( "tool_new",  "New",  "IUP_FileNew",  Icallback("new_cb")  ),
		toolbar_item( "tool_open", "Open", "IUP_FileOpen", Icallback("open_cb") ),
		toolbar_item( "tool_save", "Save", "IUP_FileSave", Icallback("save_cb"), 0 ),
		toolbar_separator(),
		toolbar_item( "tool_cut",   "Cut",   "IUP_EditCut",   Icallback("cut_cb"),   0 ),
		toolbar_item( "tool_copy",  "Copy",  "IUP_EditCopy",  Icallback("copy_cb"),  0 ),
		toolbar_item( "tool_paste", "Paste", "IUP_EditPaste", Icallback("paste_cb"), 0 ),
		toolbar_item( "tool_undo",  "Undo",  "IUP_EditUndo",  Icallback("undo_cb"),  0 ),
		toolbar_item( "tool_redo",  "Redo",  "IUP_EditRedo",  Icallback("redo_cb"),  0 ),
		toolbar_separator(),
		toolbar_item( "tool_about", "About", "IUP_MessageInfo", Icallback("about_cb") )
	})
	IupSetAttributes( toolbar, "GAP=2, MARGIN=2x2" )
	IupSetHandle( "toolbar", toolbar )

	return toolbar
end function

--**
-- Create the main status bar.
--
public function main_statusbar()

	atom status = IupLabel()
	IupSetAttribute( status, "MINSIZE", "0x23" )
	IupSetAttribute( status, "NAME", "STATUSBAR" )
	IupSetAttribute( status, "EXPAND", "HORIZONTAL" )
	IupSetAttribute( status, "PADDING", "2x4" )
	IupSetHandle( "status", status )

	return status
end function

--**
-- Create the main tabs interface.
--
public function main_tabs()

	atom tabs = IupTabs()
	IupSetAttribute( tabs, "SHOWCLOSE", "YES" )
	IupSetCallback( tabs, "TABCHANGE_CB", Icallback("tabchange_cb") )
	IupSetCallback( tabs, "TABCLOSE_CB",  Icallback("tabclose_cb")  )
	IupSetHandle( "tabs", tabs )

	return tabs
end function

--**
-- Create the main dialog.
--
public function main_dialog()

	atom menu = main_menu()
	atom vbox = IupVbox({
		main_toolbar(),
		main_tabs(),
		main_statusbar()
	})

	atom dialog = IupDialog( vbox )
--	IupSetAttribute( dialog, "SIZE", "HALFxHALF" )
	IupSetAttribute( dialog, "RASTERSIZE", "640x480" )
	IupSetAttribute( dialog, "MENU", "menu" )
	IupSetCallback( dialog, "CLOSE_CB", Icallback("closing_cb") )
	IupSetHandle( "dialog", dialog )

	atom timer = IupTimer()
	IupSetAttribute( timer, "TIME", "50" )
	IupSetAttribute( timer, "RUN", "YES" )
	IupSetCallback( timer, "ACTION_CB", Icallback("timer_cb") )
	IupSetHandle( "timer", timer )

	return dialog
end function

